--------------------------------------------------------------------------------
-- SubModule GPSTiming
-- Created   7/31/2008 4:18:24 PM
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity GPSTiming is port
   (
     CLK            : in    std_logic;
     RST            : in    std_logic;
     CNT            : in    std_logic_vector(25 downto 0);
     MASTER         : in    std_logic;

     GPSRCV         : in    std_logic_vector(31 downto 0);
     GPS_LEAP       : in    std_logic_vector(15 downto 0);
     GPS_VALID      : in    std_logic;
     GPS_LOCKED     : in    std_logic;

     UPRCV          : in    std_logic;
     GPS_GPS        : in    std_logic_vector(31 downto 0);
     LEAP           : out   std_logic_vector(15 downto 0);

     GPS            : out   std_logic_vector(31 downto 0);
     GPS_ERRC       : out   std_logic_vector(2 downto 0)
   );
end GPSTiming;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture Structure of GPSTiming is

-- Component Declarations

-- Signal Declarations
   signal tc              : std_logic;
   signal gpsmux          : unsigned (31 downto 0);
   signal gpscnt          : unsigned (31 downto 0);
   signal leapcnt         : std_logic_vector (15 downto 0);
   signal gpssync         : std_logic;
   signal errc            : unsigned (2 downto 0);

begin
   -- TC
   tc <= '1' when (CNT = B"11111111111111111111111111") else '0';

   -- GPS time logic
   gpsmux <= unsigned (GPS_GPS);
   gps_proc: process (CLK) is
   begin
      if rising_edge (CLK) then
         if RST = '1' then
            errc <= B"111";
            gpscnt <= (others => '0');
         elsif tc = '1' then
            if gpssync = '1' then
               gpscnt <= gpsmux + 1;
               leapcnt <= GPS_LEAP;
               gpssync <= '0';
               errc <= B"000";
            else
               gpscnt <= gpscnt + 1;
               if gpscnt = gpsmux then
                  leapcnt <= GPS_LEAP;
                  errc <= B"000";
               elsif errc /= B"111" then
                  errc <= errc + 1;
               end if;
            end if;
         else
            if errc = B"111" then
               if MASTER = '1' then
                  if (GPS_VALID = '1') and (GPS_LOCKED = '1') then
                     gpssync <= '1';
                  end if;
               else
                  if UPRCV = '1' then
                     gpssync <= '1';
                  end if;
               end if;
            end if;
         end if;
      end if;
   end process gps_proc;
   GPS <= std_logic_vector (gpscnt) when (MASTER = '1') else GPSRCV;
   LEAP <= leapcnt;
   GPS_ERRC <= std_logic_vector (errc);

end architecture Structure;
--------------------------------------------------------------------------------
